/**
 * Hospital Service System (HSS)
 *
 * Copyright (c) 2001/2002 Distributed Systems Research Group.
 * All rights reserved.
 *
 * For more information on Distributed Systems Research Group (DSRG),
 * please see <http://www.ics.agh.edu.pl/>.
 */

package org.dsrg.util.config;

import java.util.Properties;

import java.io.InputStream;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * The class represents boot config service
 *
 * @author <a href="mailto:bajolek@student.uci.agh.edu.pl">Maciej Bajoek</a>
 * @author <a href="mailto:artkas@student.uci.agh.edu.pl">Artur Kaszczyszyn</a>
 *
 * @version $Id: BootConfig.java,v 1.2 2002/07/24 16:31:24 maciek Exp $
 */
public class BootConfig {
	public static final String LOG_FILE_NAME_DEFAULT = "/config/log.properties";
	public static final String LOG_FILE_NAME = "monit.log.file.name";

	private static final String BOOT_CONFIG_FILE_NAME_DEFAULT =
		"/config/config.properties";

	private static final BootConfig instance = new BootConfig();
	private static boolean isInit = false;
	private static final Properties props = new Properties();

	private BootConfig() {

	}

	public static BootConfig get() {
		return BootConfig.instance;
	}

	public static void init() throws ConfigException {
		init(BOOT_CONFIG_FILE_NAME_DEFAULT);
	}

	public static void init(String bootFileName) throws ConfigException {
		if (BootConfig.isInit) {
			return;
		}

		System.out.println(
			"Initialize boot config service from file: [" + bootFileName + "]");

		InputStream is = null;
		is = BootConfig.instance.getClass().getResourceAsStream(bootFileName);
		if (is == null) {
			throw new ConfigException(
				"Boot config service file: [ " + bootFileName + "] not found");
		}

		try {
			BootConfig.props.load(is);

			StringWriter sw = new StringWriter();
			props.list(new PrintWriter(sw));

			System.out.println("Boot config service properties: " + sw);

			BootConfig.isInit = true;
		} catch (IOException e) {
			throw new ConfigException(e);
		} finally {
			if (is != null) {
				try {
					is.close();
					is = null;
				} catch (IOException e) {
					// Ignore it
				}
			}
		}
	}

	public void destroy() {
		BootConfig.isInit = false;
	}

	public static boolean isProperty(String name) {
		if (!BootConfig.isInit) {
			throw new IllegalStateException("Config service not initialize");
		}

		if (name == null) {
			throw new IllegalArgumentException("Property name not specified");
		}

		String value = BootConfig.props.getProperty(name);

		if (value == null) {
			return false;
		}

		return true;
	}

	public static String getProperty(String name) throws ConfigException {
		if (!BootConfig.isInit) {
			throw new IllegalStateException("Config service not initialized");
		}

		if (name == null) {
			throw new IllegalArgumentException("Property name not specified");
		}

		String value = BootConfig.props.getProperty(name);

		if (value == null) {
			throw new ConfigException(
				"Property [ " + name + " ] value not specified");
		}

		return value;
	}

	public static String getProperty(String name, String defaultValue) {
		try {
			return BootConfig.getProperty(name);
		} catch (Exception e) {
			return defaultValue;
		}
	}

	public static int getPropertyAsInteger(String name)
		throws ConfigException {
		try {
			String value = BootConfig.getProperty(name);

			return Integer.parseInt(value);
		} catch (NumberFormatException e) {
			throw new ConfigException(e);
		}
	}

	public static int getPropertyAsInteger(String name, int defaultValue) {
		try {
			return BootConfig.getPropertyAsInteger(name);
		} catch (Exception e) {
			return defaultValue;
		}
	}

	public static long getPropertyAsLong(String name) throws ConfigException {
		try {
			String value = BootConfig.getProperty(name);

			return Long.parseLong(value);
		} catch (NumberFormatException e) {
			throw new ConfigException(e);
		}
	}

	public static long getPropertyAsLong(String name, long defaultValue) {
		try {
			return BootConfig.getPropertyAsLong(name);
		} catch (Exception e) {
			return defaultValue;
		}
	}

	public static float getPropertyAsFloat(String name)
		throws ConfigException {
		try {
			String value = BootConfig.getProperty(name);

			return Float.parseFloat(value);
		} catch (NumberFormatException e) {
			throw new ConfigException(e);
		}
	}

	public static float getPropertyAsFloat(String name, float defaultValue) {
		try {
			return BootConfig.getPropertyAsFloat(name);
		} catch (Exception e) {
			return defaultValue;
		}
	}

	public static boolean getPropertyAsBoolean(String name)
		throws ConfigException {
		String value = BootConfig.getProperty(name);

		return Boolean.valueOf(value).booleanValue();
	}

	public static boolean getPropertyAsBoolean(
		String name,
		boolean defaultValue) {
		try {
			return BootConfig.getPropertyAsBoolean(name);
		} catch (Exception e) {
			return defaultValue;
		}
	}
}
