/**
 * Hospital Service System (HSS)
 *
 * Copyright (c) 2001/2002 Distributed Systems Research Group.
 * All rights reserved.
 *
 * For more information on Distributed Systems Research Group (DSRG),
 * please see <http://www.ics.agh.edu.pl/>.
 */


package org.dsrg.util.lookup;


import org.dsrg.util.StringUtil;
import org.dsrg.util.Assert;

import org.dsrg.exceptions.SystemException;

import javax.naming.Context;
import javax.naming.InitialContext;

import javax.ejb.EJBLocalHome;
import javax.ejb.EJBHome;

import javax.jms.Topic;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.TopicConnectionFactory;

import java.util.Map;
import java.util.HashMap;


/**
 * The class represents lookup service for the system
 *
 * @author <a href="mailto:bajolek@student.uci.agh.edu.pl">Maciej Bajoek</a>
 * @author <a href="mailto:artkas@student.uci.agh.edu.pl">Artur Kaszczyszyn</a>
 *
 * @version $Id: Lookup.java,v 1.6 2002/09/12 19:09:57 artur Exp $
 */
public class Lookup
{
	private static Lookup instance = null;
	private static boolean isInit = false;

	private Map localHomes = null;
	private Map topics = null;
	private Map queues = null;
	private Map homes = null;
	
	private QueueConnectionFactory queueConnectionFactory = null;
	private TopicConnectionFactory topicConnectionFactory = null;

	private Context context = null;

	private Lookup()
	{
		this.homes = new HashMap();
		this.topics = new HashMap();
		this.queues = new HashMap();
	}

	public static Lookup get()
	{
		if (Lookup.instance == null)
		{
			Lookup.instance = new Lookup();
			Lookup.instance.init();
		}

		return Lookup.instance;
	}

    private void init()
    {
	if (Lookup.isInit)
	{
	    return;
	}

		try
		{
			// TODO get initial context parameters form configuration
		this.context = new InitialContext();

			Lookup.isInit = true;
		}
		catch (Exception e)
		{
			throw new SystemException("Initializing lookup service ... failed", e);
		}
    }

    private void destroy()
    {
	if (!Lookup.isInit)
	{
			return;
		}

		try
		{
		if (this.context != null)
		{
		    this.context.close();
			this.context = null;
		}
		}
		catch (Exception e)
		{
			throw new SystemException("Destroying lookup service ... failed", e);
		}
		finally
		{
		Lookup.isInit = false;
		}
    }



	public EJBHome getHome(String name)
	{
		if (!Lookup.isInit)
	{
	    throw new IllegalStateException("Lookup service not initialize");
	}

		Assert.notEquals("Lookup name not specified", name, StringUtil.EMPTY);

		try
		{
			Object obj = this.homes.get(name);
			if (obj == null)
			{
				obj = this.context.lookup(name);
				this.homes.put(name, obj);
			}

			return (EJBHome) obj;
		}
		catch (Exception e)
		{
			throw new SystemException(e);
		}
	}


	public EJBLocalHome getLocalHome(String name)
	{
		if (!Lookup.isInit)
	{
	    throw new IllegalStateException("Lookup service not initialize");
	}

		Assert.notEquals("Lookup name not specified", name, StringUtil.EMPTY);

		try
		{
			Object obj = this.localHomes.get(name);
			if (obj == null)
			{
				obj = this.context.lookup(name);

				this.localHomes.put(name, obj);
			}

			return (EJBLocalHome) obj;
		}
		catch (Exception e)
		{
			throw new SystemException(e);
		}
	}

	public Topic getTopic(String name)
	{
		if (!Lookup.isInit)
	{
	    throw new IllegalStateException("Lookup service not initialize");
	}

		Assert.notEquals("Lookup name not specified", name, StringUtil.EMPTY);

		try
		{
			Object obj = this.topics.get(name);
			if (obj == null)
			{
				obj = this.context.lookup(name);

				this.topics.put(name, obj);
			}

			return (Topic) obj;
		}
		catch (Exception e)
		{
			throw new SystemException(e);
		}
	}

	public Queue getQueue(String name)
	{
		if (!Lookup.isInit)
	{
	    throw new IllegalStateException("Lookup service not initialize");
	}

		Assert.notEquals("Lookup name not specified", name, StringUtil.EMPTY);

		try
		{
			Object obj = this.queues.get(name);
			if (obj == null)
			{
				obj = this.context.lookup(name);

				this.queues.put(name, obj);
			}

			return (Queue) obj;
		}
		catch (Exception e)
		{
			throw new SystemException(e);
		}
	}

	public QueueConnectionFactory getQueueConnectionFactory(String name)
	{
		if (!Lookup.isInit)
	{
	    throw new IllegalStateException("Lookup service not initialize");
	}

		Assert.notEquals("Lookup name not specified", name, StringUtil.EMPTY);

		try
		{
			if (this.queueConnectionFactory == null)
			{
				this.queueConnectionFactory = (QueueConnectionFactory) this.context.lookup(name);
			}

			return this.queueConnectionFactory;
		}
		catch (Exception e)
		{
			throw new SystemException(e);
		}
	}

	public TopicConnectionFactory getTopicConnectionFactory(String name)
	{
		if (!Lookup.isInit)
	{
	    throw new IllegalStateException("Lookup service not initialize");
	}

		Assert.notEquals("Lookup name not specified", name, StringUtil.EMPTY);

		try
		{
			if (this.topicConnectionFactory == null)
			{
				this.topicConnectionFactory = (TopicConnectionFactory) this.context.lookup(name);
			}

			return this.topicConnectionFactory;
		}
		catch (Exception e)
		{
			throw new SystemException(e);
		}
	}
}
